@@ -1,3 +1,4 @@
+import re
 import os
 import requests
 import json
@@ -18,11 +19,19 @@ def default(o):
     if isinstance(o, (date, datetime)):
         return o.strftime("%Y-%m-%dT%H:%M:%S.000+00:00")
 
+def isValidFolder():
+    return bool(re.match(r"^/resources/labs/cvstudio/[0-9a-z-]+/(runs|apps)/[0-9a-z-]+$", os.getcwd()))
+
 class CVStudio(object):
     def __init__(self, token=None, ibm_api_key_id=None, base_url=None):
+        if base_url == None:
+            self.base_url = os.environ.get('CV_STUDIO_BASE_URL', 'https://vision.skills.network')
+        else:
+            self.base_url = base_url
+
         if token == None:
-            self.token = os.environ.get('CV_STUDIO_TOKEN')
-            if self.token == None:
+            self.token = self.getToken()
+            if self.token is None:
                 raise Exception('need to pass valid token or set CV_STUDIO_TOKEN environment variable')
         else:
             self.token = token
@@ -34,13 +43,42 @@ class CVStudio(object):
         else:
             self.ibm_api_key_id = ibm_api_key_id
 
-        if base_url == None:
-            self.base_url = os.environ.get('CV_STUDIO_BASE_URL', 'https://vision.skills.network')
-        else:
-            self.base_url = base_url
-
         self.setup()
 
+    def getToken(self):
+        token = os.environ.get('CV_STUDIO_TOKEN')
+        if token is None:
+            return None
+
+        if not isValidFolder():
+            raise Exception('Need to run from a valid folder. e.g. /resources/labs/cvstudio/<project-name>/runs/<run-name>')
+
+        rest = os.getcwd()
+        rest, itemName = os.path.split(rest)
+        rest, itemType = os.path.split(rest)
+        rest, projectName = os.path.split(rest)
+
+        data = {
+            'projectName': projectName,
+            'itemType': itemType,
+            'itemName': itemName
+        }
+
+        headers = {
+            'Authorization': 'Bearer ' + token,
+            'Content-Type': 'application/json'
+        }
+
+        x = requests.post(self.base_url + '/api/token', headers=headers, data=json.dumps(data, default=default))
+
+        if not x.ok:
+            raise("Failed to setup CV Studio client")
+
+        item = json.loads(x.text)            
+        return item['token']
+
+
+
     def setup(self):
         headers = {
             'Authorization': 'Bearer ' + self.token,
@@ -66,9 +104,9 @@ class CVStudio(object):
 
         return x
 
-    def download_file_cos(self, key):
+    def download_file_cos(self, key, folder=os.getcwd()):
         try:
-            local_file_name=os.path.join(os.getcwd(), key)
+            local_file_name=os.path.join(folder, key)
             res=self.cos.download_file(Bucket=self.bucket, Key=key, Filename=local_file_name)
         except Exception as e:
             print('Exception', e)
@@ -82,8 +120,12 @@ class CVStudio(object):
             print('Exception', e)
     
     def downloadAll(self):
+        image_folder = os.path.join(os.getcwd(), 'images')
+        if not os.path.exists(image_folder):
+                os.mkdir(image_folder)
+
         for image in tqdm(self.get_annotations()['annotations'].keys()):
-            self.download_file_cos(image)
+            self.download_file_cos(image, image_folder)
     
     def getDataset(self, transform=None):
         dataset=Dataset(annotations=self.get_annotations(), bucket_name=self.bucket, transform=transform)
@@ -91,7 +133,10 @@ class CVStudio(object):
     
     def uploadModel(self, key, details={}):
         self.upload_file_cos(key)
+        details['ready'] = True
         details['filename'] = key
+        details['filesize'] = os.stat(key).st_size
+        
         self.report(model=details)
 
     def upload_file_cos(self, filename):
@@ -166,15 +211,13 @@ class Dataset(Dataset):
             self.data=pd.read_csv(labels_filename)
         else:
             data_={'label':[],'y':[],'file_name':[],'key':[]}
-#             print('HERE')
-#             print(annotations.items())
             for key,label_dict in annotations['annotations'].items():                
                 label=label_dict[0]['label']
 
                 data_['label'].append(label)
                 data_['y'].append(labels_types.index(label))
                 data_['key'].append(key)
-                data_['file_name'].append(os.path.join(os.getcwd(),key))
+                data_['file_name'].append(os.path.join(os.getcwd(),'images',key))
 
             self.data=pd.DataFrame(data_)
             self.data.to_csv(labels_filename)
